/*****************************************************************************
 IXXAT Automation GmbH
******************************************************************************

 Project : CANCON.EXE
 File    : CANCON.C
 Summary : Demo application for the IXXAT VCI C-API.
             
 Date    : 2005-07-28

 Compiler: Microsoft VC++ 6.0

 Remarks : This demo demonstrates the following VCI features
           - adapter selection
           - controller initialisation 
           - creation of a message channel
           - transmission / reception of CAN messages
******************************************************************************
 all rights reserved
*****************************************************************************/

/*****************************************************************************
 * include files
 ****************************************************************************/
 
#include <vcinpl.h>
#include <process.h>
#include <stdio.h>
#include <conio.h>

#include "select.hpp"

/*****************************************************************************
 * structure
 ****************************************************************************/

//**** timestamp structure
typedef struct _tagDMO_TIMESTAMP
{
  DWORD dwDays;
  DWORD dwHours;  
  DWORD dwMinutes;
  DWORD dwSeconds;
  DWORD dwMilSec;
  DWORD dwMicSec;
  DWORD dwNanoSec;
}DMO_TIMESTAMP, *PDMO_TIMESTAMP;

/*****************************************************************************
 * global variables
 ****************************************************************************/

static HANDLE hDevice;  // device handle
static LONG   lCtrlNo;  // controller number
static HANDLE hCanCtl;  // controller handle 
static HANDLE hCanChn;  // channel handle

static LONG            lMustQuit            = 0; // quit flag for the receive thread
static UINT64          qwTimeStampResolNSec = 0; // time stamp precis resolution
static UINT64          qwTimeInTicks        = 0; // time stamp in ticks 
static UINT64          qwTimerOverruns      = 0; // number of timer overruns
static UINT64          qwTimeStamp          = 0; // time stamp in 100ns
static DMO_TIMESTAMP   stcTimeStamp;             // time stamp structure
static CANCAPABILITIES sCanCaps;

/*************************************************************************
 * local functions
*************************************************************************/

void SelectDevice ( BOOL fUserSelect );
void InitSocket   ( UINT32 dwCanNo );
void FinalizeApp  ( void );
void DisplayError ( HRESULT hResult );

void TransmitData ( void );
void ReceiveThread( void* Param );

HRESULT ConvertTimeStamp(UINT64         qwTimeStamp,
                         PDMO_TIMESTAMP pstcTimeStamp);

/*****************************************************************************
 Function:
  main

 Description:
  Main entry point of the application.

 Arguments:
  none

 Results:
  none
*****************************************************************************/
void main (void)
{
  // a varialble to get the last pressed key
  BYTE bChar;


  printf(" >>>> VCI - C-API Example V2.0 <<<<\n");
  printf("\n Initializes the CAN with 125 kBaud");
  printf("\n Shows all received messages");
  printf("\n 't'-key send a message with ID 100H");
  printf("\n 'Esc'-key quit the application\n\n");

  printf(" Select Adapter...\r");  
  SelectDevice( TRUE );
  printf(" Select Adapter.......... OK !\n\n");

  printf(" Initialize CAN...\r");
  InitSocket( lCtrlNo );
  printf(" Initialize CAN............ OK !\n\n");
  
  //
  // start the receive thread
  //
  _beginthread( ReceiveThread, 0, NULL);

  //
  // wait for keyboard hit transmit  CAN-Messages cyclically
  //
  while( 1 )
  { 
    // wait for the user to press a key
    bChar = _getch();

    // when the key is 't' ort 'T' the send a CAN message
    if ( (bChar == 't') || (bChar == 'T') )
      TransmitData();  

    // when the user press the ESC key then end the program
    if (bChar == VK_ESCAPE)
      break;

    Sleep(1);
  } 

  //
  // tell receive thread to quit
  //
  InterlockedExchange(&lMustQuit, 1);

  printf(" Free VCI - Resources...\r");
  FinalizeApp();
  printf(" Free VCI - Resources........ OK !\n\n");
}

/*****************************************************************************
 Function:
  SelectDevice

 Description:
  Selects the first CAN adapter.

 Arguments:
  fUserSelect -> If this parameter is set to TRUE the functions display
                 a dialog box which allows the user to select the device.

 Results:
  none
*****************************************************************************/
void SelectDevice( BOOL fUserSelect )
{
  HRESULT hResult; // error code

  if (fUserSelect == FALSE)
  {
    HANDLE        hEnum;   // enumerator handle
    VCIDEVICEINFO sInfo;   // device info

    //
    // open the device list
    //
    hResult = vciEnumDeviceOpen(&hEnum);

    //
    // retrieve information about the first
    // device within the device list
    //
    if (hResult == VCI_OK)
    {
      hResult = vciEnumDeviceNext(hEnum, &sInfo);
    }

    //
    // close the device list (no longer needed)
    //
    vciEnumDeviceClose(hEnum);

    //
    // open the device
    //
    if (hResult == VCI_OK)
    {
      hResult = vciDeviceOpen(sInfo.VciObjectId, &hDevice);
    }

    //
    // always select controller 0
    //
    lCtrlNo = 0;
  }
  else
  {
    //
    // open a device selected by the user
    //
    hResult = SocketSelectDlg(NULL, VCI_BUS_CAN, &hDevice, &lCtrlNo);
  }

  DisplayError(hResult);
}

/*************************************************************************
 Function: 
  InitSocket

 Description : 
  Opens the specified socket, creates a message channel, initializes
  and starts the CAN controller.

 Arguments: 
  dwCanNo -> Number of the CAN controller to open.

 Results:
  none

 Remarks:
  If <dwCanNo> is set to 0xFFFFFFFF, the function shows a dialog box
  which allows the user to select the VCI device and CAN controller.
*************************************************************************/
void InitSocket(UINT32 dwCanNo)
{
  HRESULT hResult;

  //
  // create a message channel
  //
  if (hDevice != NULL)
  {
    //
    // create and initialize a message channel
    //
    hResult = canChannelOpen(hDevice, dwCanNo, FALSE, &hCanChn);

    //
    // initialize the message channel
    //
    if (hResult == VCI_OK)
    {
      UINT16 wRxFifoSize  = 1024;
      UINT16 wRxThreshold = 1;
      UINT16 wTxFifoSize  = 128;
      UINT16 wTxThreshold = 1;

      hResult = canChannelInitialize( hCanChn,
                                      wRxFifoSize, wRxThreshold,
                                      wTxFifoSize, wTxThreshold);
    }

    //
    // activate the CAN channel
    //
    if (hResult == VCI_OK)
    {
      hResult = canChannelActivate(hCanChn, TRUE);
    }

    //
    // open the CAN controller
    //
    if (hResult == VCI_OK)
    {
      hResult = canControlOpen(hDevice, dwCanNo, &hCanCtl);
      // this function fails if the controller is in use
      // by another application.
    }

    //
    // calculate the time resolution
    //
    if (hResult == VCI_OK)
	{ 
      // calulate the time resolution in nano seconds
      hResult = canControlGetCaps(hCanCtl, &sCanCaps);	  
      qwTimeStampResolNSec = ((UINT64) 1000000000 * sCanCaps.dwTscDivisor) / sCanCaps.dwClockFreq;
    }

    //
    // initialize the CAN controller
    //
    if (hResult == VCI_OK)
    { 
      hResult = canControlInitialize(hCanCtl, CAN_OPMODE_STANDARD,
                                     CAN_BT0_125KB, CAN_BT1_125KB);
    }

    //
    // set the acceptance filter
    //
    if (hResult == VCI_OK)
    { 
       hResult = canControlSetAccFilter( hCanCtl, FALSE,
                                         CAN_ACC_CODE_ALL, CAN_ACC_MASK_ALL);
    }

    //
    // start the CAN controller
    //
    if (hResult == VCI_OK)
    {
      hResult = canControlStart(hCanCtl, TRUE);
    }	
  }
  else
  {
    hResult = VCI_E_INVHANDLE;
  }

  DisplayError(hResult);
}

/*************************************************************************
 Function: 
  TransmitData

 Description: 
  Transmits a CAN message with ID 0x100.

 Arguments: 
  none

 Results:
  none
*************************************************************************/
void TransmitData(void)
{
  HRESULT hResult;
  CANMSG  sCanMsg;
  UINT8   i;

  sCanMsg.dwTime   = 0;
  sCanMsg.dwMsgId  = 0x100;    // CAN message identifier

  sCanMsg.uMsgInfo.Bytes.bType  = CAN_MSGTYPE_DATA;
  sCanMsg.uMsgInfo.Bytes.bFlags = CAN_MAKE_MSGFLAGS(8,0,0,0,0);

  for (i = 0; i < sCanMsg.uMsgInfo.Bits.dlc; i++ )
  {
    sCanMsg.abData[i] = i;
  }

  // write the CAN message into the transmit FIFO
  hResult = canChannelSendMessage(hCanChn, INFINITE, &sCanMsg);
}

/*************************************************************************
 Function: 
  ReceiveThread

 Description: 
  Receive thread.

 Arguments: 
  none

 Results:
  none
*************************************************************************/
void ReceiveThread( void* Param )
{
  HRESULT hResult;
  CANMSG  sCanMsg;
  
  while ( lMustQuit == 0 )
  {
    // read a CAN message from the receive FIFO

    hResult = canChannelReadMessage(hCanChn, 100, &sCanMsg);

    if (hResult == VCI_OK)
    {
      if (sCanMsg.uMsgInfo.Bytes.bType == CAN_MSGTYPE_DATA)
      {
        //
        // show data frames
        //
        if (sCanMsg.uMsgInfo.Bits.rtr == 0)
        {
          UINT8 j;		  

		  // calculate the time stamp in ticks
		  qwTimeInTicks = (UINT64)(sCanMsg.dwTime) + (UINT64)(qwTimerOverruns << 32);

		  // calculate the time stamp in nano seconds.
          qwTimeStamp = qwTimeStampResolNSec * qwTimeInTicks;
 		  ConvertTimeStamp((qwTimeStamp/100), &stcTimeStamp);

		  // output the time stamp and CAN message
          printf("\nTime: %05u %02u:%02u:%02u.%03u.%03u.%03u ID: %3X DLC: %1u Data:",
                  stcTimeStamp.dwDays,
                  stcTimeStamp.dwHours,  
                  stcTimeStamp.dwMinutes,
                  stcTimeStamp.dwSeconds,
                  stcTimeStamp.dwMilSec,
                  stcTimeStamp.dwMicSec,
                  stcTimeStamp.dwNanoSec,                  
                  sCanMsg.dwMsgId,
				  sCanMsg.uMsgInfo.Bits.dlc);

          for (j = 0; j < sCanMsg.uMsgInfo.Bits.dlc; j++)
          {
            printf(" %.2X", sCanMsg.abData[j]);
          }
 		  // optional output of time stamp in ticks and numbero of time stamp overruns 
		  //printf("Time: %u, TimerOverruns: %u", sCanMsg.dwTime, qwTimerOverruns);

        }
        else
        {
          printf("\nTime: %10u ID: %3X  DLC: %1u  Remote Frame",
                 sCanMsg.dwTime,
                 sCanMsg.dwMsgId,
                 sCanMsg.uMsgInfo.Bits.dlc);
        }
      }
      else if (sCanMsg.uMsgInfo.Bytes.bType == CAN_MSGTYPE_INFO)
      {
        //
        // show informational frames
        //
        switch (sCanMsg.abData[0])
        {
          case CAN_INFO_START: printf("\nCAN started..."); break;
          case CAN_INFO_STOP : printf("\nCAN stoped...");  break;
          case CAN_INFO_RESET: printf("\nCAN reseted..."); break;
        }
      }	  
      else if (sCanMsg.uMsgInfo.Bytes.bType == CAN_MSGTYPE_TIMEOVR)
	    {
		    // store the timer overruns
		    qwTimerOverruns += sCanMsg.dwMsgId;
	    }

      else if (sCanMsg.uMsgInfo.Bytes.bType == CAN_MSGTYPE_ERROR)
      {
        //
        // show error frames
        //
        switch (sCanMsg.abData[0])
        {
          case CAN_ERROR_STUFF: printf("\nstuff error...");          break; 
          case CAN_ERROR_FORM : printf("\nform error...");           break; 
          case CAN_ERROR_ACK  : printf("\nacknowledgment error..."); break;
          case CAN_ERROR_BIT  : printf("\nbit error...");            break; 
          case CAN_ERROR_CRC  : printf("\nCRC error...");            break; 
          case CAN_ERROR_OTHER:
          default             : printf("\nother error...");          break;
        }
      }
    }
  }

  _endthread();
}

/*************************************************************************
 Function: 
  FinalizeApp

 Description: 
  Finalizes the application

 Arguments: 
  none
  
 Results:
  none
*************************************************************************/
void FinalizeApp()
{
  //
  // close all open handles

  canChannelClose(hCanChn);
  canControlClose(hCanCtl);

  vciDeviceClose(hDevice);
}

/*************************************************************************
 Function: 
  DisplayError

 Description : 
  This function displays a message box for the specified error code.

 Arguments : 
  hResult -> Error code or -1 to display the error code returned
             by GetLastError().

 Results:
  none
*************************************************************************/
void DisplayError(HRESULT hResult)
{
  char szError[VCI_MAX_ERRSTRLEN];

  if (hResult != NO_ERROR)
  {
    if (hResult == -1)
      hResult = GetLastError();

    szError[0] = 0;
    vciFormatError(hResult, szError, sizeof(szError));
    MessageBox(NULL, szError, "VCI Demo", MB_OK | MB_ICONSTOP);
  }
}

/*****************************************************************************
 Function:
  ConvertTimeStamp

 Description:
  Converts the UINT64 time stamp in 100ns into the time stamp structure
  days:hours:min:sec:msec:nsec 

 Arguments:
  qwTimeStamp   -> time stamp to convert
  pstcTimeStamp <- pointer to structure which receives the time stamp

 Results:
  The function returns S_OK on success, otherwise an error code

 Anotation:
  The largest timestamp supported is: 21474836479999999
  which is in the timestamp structure:
  days:         24855
  hours:        3
  minutes:      14
  secounds:     7
  microseconds: 999999
  nanoseconds:  900

*****************************************************************************/
HRESULT ConvertTimeStamp(UINT64         qwTimeStamp,
                         PDMO_TIMESTAMP pstcTimeStamp)
{
  HRESULT hResult    = S_OK;
  ldiv_t  div_result = {0};
  UINT64  qwTime     = 0;
  DWORD   dwTime     = 0;
  
  //*** Check Arguments
  if(pstcTimeStamp == NULL)
    return E_INVALIDARG;
  if(qwTimeStamp > 21474836479999999)
    return ERROR_CALL_NOT_IMPLEMENTED;

  //qwTimeStamp = 21474836479999999;

  //*** Convert to seconds       ms   s   ns
  dwTime = (DWORD)(qwTimeStamp / (1000*1000*10));

  //*** ms
  qwTime = ((UINT64)dwTime)*(1000);
  pstcTimeStamp->dwMilSec  = (DWORD)(qwTimeStamp/(1000*10) - qwTime);

  //*** s
  qwTime *= 1000;
  pstcTimeStamp->dwMicSec  = (DWORD)(qwTimeStamp/10 - qwTime - pstcTimeStamp->dwMilSec*1000);

  //*** ns
  qwTime *= 10;
  pstcTimeStamp->dwNanoSec = (DWORD)(qwTimeStamp - qwTime - pstcTimeStamp->dwMilSec*1000*10 - pstcTimeStamp->dwMicSec*10)*100;

  //*** days
  div_result = ldiv(dwTime, (24*60*60)); 
  pstcTimeStamp->dwDays = div_result.quot;
  dwTime = div_result.rem;

  //*** hours
  div_result = ldiv(dwTime, (60*60)); 
  pstcTimeStamp->dwHours = div_result.quot;
  dwTime = div_result.rem;

  //*** minutes
  div_result = ldiv(dwTime, (60)); 
  pstcTimeStamp->dwMinutes = div_result.quot;
  dwTime = div_result.rem;

  //*** seconds
  pstcTimeStamp->dwSeconds = dwTime;

  return hResult;
}


